#include <stdlib.h>

/* Drawing related stuff and definitions */
#include <fxcg/display.h>

/* Keyboard management stuff and definitons */
#include <fxcg/keyboard.h>

/* Timer stuff*/
#include <fxcg/system.h>


#define BORDER_WIDTH 20
#define STICK_WIDTH 10
#define STICK_HEIGHT 25
#define BALL_WIDTH 10

int PRGM_GetKey(void)
{
  unsigned char buffer[12];
  PRGM_GetKey_OS( buffer );
  return ( buffer[1] & 0x0F ) * 10 + ( ( buffer[2] & 0xF0 ) >> 4 );
}


/* The following structure contains the information*/
struct game {
    int player_1_y, player_1_vy;
    int player_2_y, player_2_vy;

    int ball_x, ball_y;
    int ball_vx, ball_vy;

    int player_1_score, player_2_score;
};

/* current_game will be what the whole program will play on */
struct game current_game;

/* Function that just resets the current game */
void init_game()
{
    current_game.player_1_y = LCD_HEIGHT_PX/2;
    current_game.player_1_vy = 0;
    current_game.player_2_y = LCD_HEIGHT_PX/2;
    current_game.player_2_vy = 0;

    current_game.ball_x = LCD_WIDTH_PX/2;
    current_game.ball_y = LCD_HEIGHT_PX/2;
    current_game.ball_vx = 3;
    current_game.ball_vy = 5;

    current_game.player_1_score = 0;
    current_game.player_2_score = 0;
}

void draw_filled_rectangle(int x, int y, int width, int height, color_t color) 
{
    color_t *VRAM = GetVRAMAddress();

    for(int j = y; j < y+height; j++) {
        for(int i = x; i < x + width; i++) {
            VRAM[j*LCD_WIDTH_PX + i] = color;
        }
    }
}

/* graphic engine */
void draw_game()
{
    /* fill the whole (3) VRAM */
    Bdisp_Fill_VRAM(COLOR_DARKBLUE, 3);

    /* draw the ball */
    draw_filled_rectangle(current_game.ball_x, current_game.ball_y, 
                            BALL_WIDTH, BALL_WIDTH, COLOR_RED);
    
    /* draw sticks */
    draw_filled_rectangle(BORDER_WIDTH, current_game.player_1_y - STICK_HEIGHT/2,
                                        STICK_WIDTH, STICK_HEIGHT, COLOR_BLUE);
    draw_filled_rectangle(LCD_WIDTH_PX - BORDER_WIDTH - STICK_WIDTH, 
                            current_game.player_2_y - STICK_HEIGHT/2,
                                        STICK_WIDTH, STICK_HEIGHT, COLOR_GREEN);

    /* transfer the VRAM to the screen */
    Bdisp_PutDisp_DD();
}

/* game engine */
void update_game()
{
    int new_p1_y = current_game.player_1_y + current_game.player_1_vy;
    int new_p2_y = current_game.player_2_y + current_game.player_2_vy;

    if(new_p1_y > STICK_HEIGHT/2 && new_p1_y < LCD_HEIGHT_PX - STICK_HEIGHT/2) {
        current_game.player_1_y = new_p1_y;
    } else current_game.player_1_y = (new_p1_y > STICK_HEIGHT/2)*(LCD_HEIGHT_PX-STICK_HEIGHT) + STICK_HEIGHT/2+1;
   
    if(new_p2_y > STICK_HEIGHT/2 && new_p2_y < LCD_HEIGHT_PX - STICK_HEIGHT/2) {
        current_game.player_2_y = new_p2_y;
    } else current_game.player_2_y = (new_p2_y > STICK_HEIGHT/2)*(LCD_HEIGHT_PX-STICK_HEIGHT) + STICK_HEIGHT/2 + 1;

    int p1_y = current_game.player_1_y;
    int p2_y = current_game.player_2_y;

    int new_ball_x = current_game.ball_x + current_game.ball_vx;
    int new_ball_y = current_game.ball_y + current_game.ball_vy;

    if ((new_ball_x - BALL_WIDTH/2 < BORDER_WIDTH+STICK_WIDTH && 
            new_ball_y - BALL_WIDTH/2 < p1_y + STICK_HEIGHT/2 && 
            new_ball_y + BALL_WIDTH/2 > p1_y - STICK_HEIGHT/2) ||
        (new_ball_x - BALL_WIDTH/2 < 0) ||
        (new_ball_x + BALL_WIDTH/2 > LCD_WIDTH_PX-BORDER_WIDTH-STICK_WIDTH &&
            new_ball_y - BALL_WIDTH/2 < p2_y + STICK_HEIGHT/2 &&
            new_ball_y + BALL_WIDTH/2 > p2_y - STICK_HEIGHT/2) ||
        (new_ball_x + BALL_WIDTH/2 > LCD_WIDTH_PX)) {
            current_game.ball_vx = -current_game.ball_vx;
        }

    if ((new_ball_y - BALL_WIDTH/2 < 0) || 
        (new_ball_y + BALL_WIDTH/2 > LCD_HEIGHT_PX)) {
            current_game.ball_vy = -current_game.ball_vy;
        }

    current_game.ball_x = new_ball_x;
    current_game.ball_y = new_ball_y;
}

int main()
{
    int key;
    int timer_slot;
    
    /* first, get a clean game */
    init_game();

    /* setup the timer */
    timer_slot = Timer_Install(0, update_game, 32);
    Timer_Start(timer_slot);
    do {
        asm("sleep");
        //key = PRGM_GetKey(); 
        GetKey(&key);
        if(key == KEY_PRGM_UP) {
            current_game.player_1_vy = -4;
        } else if(key == KEY_PRGM_DOWN) {
            current_game.player_1_vy = 4;
        } else {
            current_game.player_1_vy = 0;
        }

        draw_game();
        
    } while(key != KEY_PRGM_MENU);

    Timer_Stop(timer_slot);
    Timer_Deinstall(timer_slot);
    
    DisplayMessageBox((unsigned char*)"GGWP");
    return 0;
}
